@@ -1,9 +1,9 @@
 """Typer, build great CLIs. Easy to code. Based on Python type hints."""
 
-__version__ = "0.0.6"
+__version__ = "0.0.9"
 
-from click.exceptions import Abort, Exit  # noqa
-from click.termui import (  # noqa
+from click.exceptions import Abort, Exit
+from click.termui import (
     clear,
     confirm,
     echo_via_pager,
@@ -18,7 +18,7 @@ from click.termui import (  # noqa
     style,
     unstyle,
 )
-from click.utils import (  # noqa
+from click.utils import (
     echo,
     format_filename,
     get_app_dir,
@@ -27,6 +27,7 @@ from click.utils import (  # noqa
     open_file,
 )
 
-from .main import Typer, run  # noqa
-from .models import BinaryFileRead, BinaryFileWrite, Context, TextFile  # noqa
-from .params import Argument, Option  # noqa
+from . import colors
+from .main import Typer, run
+from .models import Context, FileBinaryRead, FileBinaryWrite, FileText, FileTextWrite
+from .params import Argument, Option
