@@ -11,17 +11,18 @@ import click
 from .models import (
     AnyType,
     ArgumentInfo,
-    BinaryFileRead,
-    BinaryFileWrite,
     CommandFunctionType,
     CommandInfo,
     Default,
     DefaultPlaceholder,
+    FileBinaryRead,
+    FileBinaryWrite,
+    FileText,
+    FileTextWrite,
     NoneType,
     OptionInfo,
     ParameterInfo,
     Required,
-    TextFile,
     TyperInfo,
 )
 
@@ -225,6 +226,7 @@ def get_command(typer_instance: Typer) -> click.Command:
         click_install_param, click_show_param = get_install_completion_arguments()
     if (
         typer_instance.registered_callback
+        or typer_instance.info.callback
         or typer_instance.registered_groups
         or len(typer_instance.registered_commands) > 1
     ):
@@ -241,7 +243,7 @@ def get_command(typer_instance: Typer) -> click.Command:
             click_command.params.append(click_install_param)
             click_command.params.append(click_show_param)
         return click_command
-    assert False, "Could not get a command for this Typer instance"
+    assert False, "Could not get a command for this Typer instance"  # pragma no cover
 
 
 def get_group_name(typer_info: TyperInfo) -> Optional[str]:
@@ -260,33 +262,44 @@ def get_group_name(typer_info: TyperInfo) -> Optional[str]:
 
 
 def solve_typer_info_help(typer_info: TyperInfo) -> str:
-    # Priority 1a: Value was set in app.add_typer()
+    # Priority 1: Explicit value was set in app.add_typer()
     if not isinstance(typer_info.help, DefaultPlaceholder):
         return inspect.cleandoc(typer_info.help or "")
-    # Priority 1b: Value was set in app.add_typer(), in callback docstring
+    # Priority 2: Explicit value was set in sub_app.callback()
+    try:
+        callback_help = typer_info.typer_instance.registered_callback.help
+        if not isinstance(callback_help, DefaultPlaceholder):
+            return inspect.cleandoc(callback_help or "")
+    except AttributeError:
+        pass
+    # Priority 3: Explicit value was set in sub_app = typer.Typer()
+    try:
+        instance_help = typer_info.typer_instance.info.help
+        if not isinstance(instance_help, DefaultPlaceholder):
+            return inspect.cleandoc(instance_help or "")
+    except AttributeError:
+        pass
+    # Priority 4: Implicit inference from callback docstring in app.add_typer()
     if typer_info.callback:
         doc = inspect.getdoc(typer_info.callback)
         if doc:
             return doc
+    # Priority 5: Implicit inference from callback docstring in @app.callback()
     try:
-        # Priority 2a: Value was set in @subapp.callback()
-        doc = typer_info.typer_instance.registered_callback.help
-        if not isinstance(doc, DefaultPlaceholder):
-            return inspect.cleandoc(doc or "")
-        # Priority 2b: Value was set in @subapp.callback(), in callback docstring
-        doc = inspect.getdoc(typer_info.typer_instance.registered_callback.callback)
-        if doc:
-            return doc
+        callback = typer_info.typer_instance.registered_callback.callback
+        if not isinstance(callback, DefaultPlaceholder):
+            doc = inspect.getdoc(callback or "")
+            if doc:
+                return doc
     except AttributeError:
         pass
+    # Priority 6: Implicit inference from callback docstring in typer.Typer()
     try:
-        # Priority 3a: Value set in subapp = typer.Typer()
-        instance_value = typer_info.typer_instance.info.help
-        if not isinstance(instance_value, DefaultPlaceholder):
-            return inspect.cleandoc(instance_value or "")
-        doc = inspect.getdoc(typer_info.typer_instance.callback)
-        if doc:
-            return doc
+        instance_callback = typer_info.typer_instance.info.callback
+        if not isinstance(instance_callback, DefaultPlaceholder):
+            doc = inspect.getdoc(instance_callback)
+            if doc:
+                return doc
     except AttributeError:
         pass
     # Value not set, use the default
@@ -295,7 +308,6 @@ def solve_typer_info_help(typer_info: TyperInfo) -> str:
 
 def solve_typer_info_defaults(typer_info: TyperInfo) -> TyperInfo:
     values: Dict[str, Any] = {}
-    values["help"] = solve_typer_info_help(typer_info)
     name = None
     for name, value in typer_info.__dict__.items():
         # Priority 1: Value was set in app.add_typer()
@@ -324,6 +336,7 @@ def solve_typer_info_defaults(typer_info: TyperInfo) -> TyperInfo:
         values[name] = value.value
     if values["name"] is None:
         values["name"] = get_group_name(typer_info)
+    values["help"] = solve_typer_info_help(typer_info)
     return TyperInfo(**values)
 
 
@@ -449,6 +462,13 @@ def generate_enum_convertor(enum: Type[Enum]) -> Callable:
     return convertor
 
 
+def generate_iter_convertor(convertor: Callable[[Any], Any]) -> Callable:
+    def internal_convertor(value: Any) -> List[Any]:
+        return [convertor(v) for v in value]
+
+    return internal_convertor
+
+
 def get_callback(
     *,
     callback: Optional[Callable] = None,
@@ -526,7 +546,15 @@ def get_click_type(
             allow_dash=parameter_info.allow_dash,
             path_type=parameter_info.path_type,
         )
-    elif lenient_issubclass(annotation, TextFile):
+    elif lenient_issubclass(annotation, FileTextWrite):
+        return click.File(
+            mode=parameter_info.mode or "w",
+            encoding=parameter_info.encoding,
+            errors=parameter_info.errors,
+            lazy=parameter_info.lazy,
+            atomic=parameter_info.atomic,
+        )
+    elif lenient_issubclass(annotation, FileText):
         return click.File(
             mode=parameter_info.mode or "r",
             encoding=parameter_info.encoding,
@@ -534,7 +562,7 @@ def get_click_type(
             lazy=parameter_info.lazy,
             atomic=parameter_info.atomic,
         )
-    elif lenient_issubclass(annotation, BinaryFileRead):
+    elif lenient_issubclass(annotation, FileBinaryRead):
         return click.File(
             mode=parameter_info.mode or "rb",
             encoding=parameter_info.encoding,
@@ -542,7 +570,7 @@ def get_click_type(
             lazy=parameter_info.lazy,
             atomic=parameter_info.atomic,
         )
-    elif lenient_issubclass(annotation, BinaryFileWrite):
+    elif lenient_issubclass(annotation, FileBinaryWrite):
         return click.File(
             mode=parameter_info.mode or "wb",
             encoding=parameter_info.encoding,
@@ -555,7 +583,7 @@ def get_click_type(
             [item.value for item in annotation],
             case_sensitive=parameter_info.case_sensitive,
         )
-    raise RuntimeError(f"Type not yet supported: {annotation}")
+    raise RuntimeError(f"Type not yet supported: {annotation}")  # pragma no cover
 
 
 def lenient_issubclass(
@@ -609,15 +637,15 @@ def get_click_param(
         # Handle Tuples and Lists
         if lenient_issubclass(origin, List):
             main_type = main_type.__args__[0]
-            assert (
-                "__origin__" not in main_type
+            assert not getattr(
+                main_type, "__origin__", None
             ), "List types with complex sub-types are not currently supported"
             is_list = True
         elif lenient_issubclass(origin, Tuple):  # type: ignore
             types = []
-            for type_ in origin.__args__:
-                assert (
-                    "__origin__" not in type_
+            for type_ in main_type.__args__:
+                assert not getattr(
+                    type_, "__origin__", None
                 ), "Tuple types with complex sub-types are not currently supported"
                 types.append(
                     get_click_type(annotation=type_, parameter_info=parameter_info)
@@ -632,6 +660,9 @@ def get_click_param(
         convertor = param_path_convertor
     if lenient_issubclass(main_type, Enum):
         convertor = generate_enum_convertor(main_type)
+    if convertor and is_list:
+        convertor = generate_iter_convertor(convertor)
+        # TODO: handle recursive conversion for tuples
     if isinstance(parameter_info, OptionInfo):
         if main_type is bool and not (parameter_info.is_flag is False):
             is_flag = True
@@ -682,8 +713,6 @@ def get_click_param(
         )
     elif isinstance(parameter_info, ArgumentInfo):
         param_decls = [param.name]
-        if parameter_info.param_decls:
-            param_decls.extend(parameter_info.param_decls)
         nargs = None
         if is_list:
             nargs = -1
@@ -705,7 +734,7 @@ def get_click_param(
             ),
             convertor,
         )
-    assert False, "A click.Parameter should be returned"
+    assert False, "A click.Parameter should be returned"  # pragma no cover
 
 
 def run(function: Callable) -> Any:
